# -*-perl-*-

# $Id: tkgui.pl,v 1.3 2008/04/06 14:49:45 maeda Exp $

# Copyright 2008 by Kaoru Maeda <kaoru.maeda@gmail.com>
#
# This program is free software; you may redistribute it and/or modify it
# under the same terms as Perl itself.

# Encoding: このファイルはutf8

use utf8;
use strict;
use Encode;

use threads;
use threads::shared;
no warnings 'threads';

our $C;

my ($MW, $List, $ProgressBar, $Message, $Ok, $Cancel);
my ($Srcdir, $Dstdir);
my @Formats;
my %Format_index;
my @Descs;

my $thr;
my $params :shared;
my $files_total :shared;
my $files_done :shared;

my @Disable_when_running;

sub tkgui {
    $params = undef;
    $thr = threads->new(\&start_gui);
    watch_gui();
}

# main thread
sub watch_gui {
    our $progressCB;
    our $format;
    our $mode;
    while (1) {
	if (defined($params)) {
	    $progressCB = \&progress;
	    $format = ${$params->[0]};
	    $mode = 'standalone';
	    @ARGV = (${$params->[1]}, ${$params->[2]});
	    $params = undef;
	    main();
	    #test();
	}
    } continue {
	sleep 1;
    }
}

sub progress {
    my ($total, $done) = @_;
    $files_total = $total;
    $files_done = $done;
}

sub test {
    sleep(1);
    $|=1;
    for my $i (0..12) {
	print "test\n";
	progress(12, $i);
	sleep(1);
    }
}

# GUI thread

sub start_gui {
    require Tk;
    prepare();
    Tk->MainLoop();
}

sub format_formats {
    @Formats =
	sort {$C->{Format}{$b}{NoVideo} <=> $C->{Format}{$a}{NoVideo}
		  || $a cmp $b}
	keys %{$C->{Format}};
    for (my $i = 0; $i < @Formats; $i++) {
	my $f = $Formats[$i];
	$Format_index{$f} = $i;
	$Descs[$i] = utf8(sprintf("%-8.8s  %s", $f, $C->{Format}{$f}{Description}));
    }
}

sub prepare {
    require Tk;
    require Tk::Dialog;
    require Tk::FileSelect;
    require Tk::ProgressBar;

    format_formats();

    $MW = Tk::MainWindow->new;
    $MW->title('Psyenc');

    for my $i (['変換元',\$Srcdir], ['保存先',\$Dstdir]) {
	my $f = $MW->Frame;
	my $lab = $f->Label(-text => "$i->[0] フォルダ: ", -anchor => 'e');
	my $ent = $f->Entry(-width => 40);
	my $but = $f->Button(-text => "選択 ...",
			     -command => sub { fileDialog($MW, $ent, $i->[0])});
	$lab->pack(-side => 'left');
	$ent->pack(-side => 'left',-expand => 'yes', -fill => 'x');
	$but->pack(-side => 'left');
	push(@Disable_when_running, $ent, $but);
	$f->pack(-fill => 'x', -padx => '1c', -pady => 3);
	${$i->[1]} = $ent;
    }

    my $f = $MW->Frame;
    $List = $f->Scrolled(qw(Listbox -setgrid 1 -height 12 -scrollbars e));
    $List->pack(qw/-side left -expand yes -fill both -pady 2m/);
    $List->focus;
    $List->insert(0, @Descs);
    #print STDERR "format=$format index = $Format_index{$format}\n";
    $List->activate(0);
    $f->pack(-fill => 'x', -padx => '1c', -pady => 3);
    push(@Disable_when_running, $List);

    $f = $MW->Frame;
    $Message = $f->Label(-text => '')->pack(-fill => 'x', -padx => '2c');
    $ProgressBar =
	$f->ProgressBar(qw(-width 20 -length 200 -borderwidth 2 -blocks 1),
			-colors => [0, 'green'])
	    ->pack(qw(-fill both -padx 10 -pady 10));
    $f->pack(-fill => 'y', -padx => '2c', -pady => '1m');

    $f = $MW->Frame;
    $Ok = $f->Button(-text => '実行', -command => sub { runPsyenc() });
    $Cancel = $f->Button(-text => '終了', -command => sub { exit(0) });
    $Ok->pack(qw/-side left -fill x -pady 2m/);
    $Cancel->pack(qw/-side right -fill x -pady 2m/);
    $f->pack(-fill => 'x', -padx => '2c' );
    push(@Disable_when_running, $Ok);

    $MW->repeat(100, \&update);
}

sub disable {
    for my $i (@Disable_when_running) {
	$i->configure(-state => 'disabled');
    }
}
sub enable {
    for my $i (@Disable_when_running) {
	$i->configure(-state => 'normal');
    }
}

sub fileDialog {
    my ($w, $ent, $op) = @_;
    my $initdir = $ent->get;
    $initdir =~ s[/][\\]g;
    $initdir = sjis($initdir);

    my $dir = $w->chooseDirectory(-initialdir => $initdir,
				  -title => sjis("$op フォルダを選んでください"),
				  -mustexist => 1);
    if (defined $dir and $dir ne '') {
	$dir =~ s[/][\\]g;
	$dir = utf8($dir);
	$dir =~ s[\\][/]g;
	$ent->delete(0, 'end');
	$ent->insert(0, $dir);
	$ent->xview('end');
    }
    $files_done = $files_total = 0;
    $Message->configure(-text => '');
}

sub runPsyenc {
    my $src_u = $Srcdir->get;
    my $dst_u = $Dstdir->get;
    my $src_s = sjis($src_u);
    my $dst_s = sjis($dst_u);

    my $error;
    unless (-d $src_s) {
	$error .= "変換元フォルダを指定してください\n";
    }
    unless (-d $dst_s) {
	$error .= "保存先フォルダを指定してください\n";
    }
    if ($error) {
	$MW->messageBox(-title => 'エラー',
			-icon => 'error',
			-message => $error,
			-default => 'OK');
	return;
    }


    $MW->Busy;
    disable();
    $Message->configure(-text => '変換中...');
    $Cancel->configure(-text => '中止');
    $files_done = 0;
    $files_total = 0;
    $ProgressBar->value(0);

    my $fmt = $List->curselection->[0];
    my $f = $Formats[$fmt];
    my $p = &share([]);
    $p->[0] = share($f);
    $p->[1] = share($src_s);
    $p->[2] = share($dst_s);
    $params = $p;
}

sub update {
    my $p = $files_total > 0 ? int(100*$files_done/$files_total) : 0;
    $ProgressBar->value($p);
    if ($files_total > 0 && $p < 100) {
	$Message->configure(-text => "変換中... $files_done/$files_total");
    }
    if ($p == 100) {
	$Message->configure(-text => '変換中... 完了');
	$MW->Unbusy;
	enable();
	$Cancel->configure(-text => '終了');
    }
}

1;
